@@ -8,7 +8,7 @@
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
-# Copyright (c) 2004-2005 Kenneth J. Pronovici.
+# Copyright (c) 2004-2006 Kenneth J. Pronovici.
 # All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
@@ -27,7 +27,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: filesystem.py 932 2005-12-13 21:31:28Z pronovic $
+# Revision : $Id: filesystem.py 986 2006-06-24 19:23:22Z pronovic $
 # Purpose  : Provides filesystem-related objects.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -107,6 +107,10 @@ class FilesystemList(list):
    bounded at front and back by the beginning and end of the string, i.e. they
    are treated as if they begin with C{^} and end with C{$}.
 
+   @note: Some platforms, like Windows, do not support soft links.  On those
+   platforms, the ignore-soft-links flag can be set, but it won't do any good
+   because the operating system never reports a file as a soft link.
+
    @sort: __init__, addFile, addDir, addDirContents, removeFiles, removeDirs,
           removeLinks, removeMatch, removeInvalid, normalize, validate, 
           excludeFiles, excludeDirs, excludeLinks, excludePaths, 
@@ -323,14 +327,15 @@ class FilesystemList(list):
       logger.debug("Added directory to list: [%s]" % path)
       return 1
 
-   def addDirContents(self, path):
+   def addDirContents(self, path, recursive=True):
       """
       Adds the contents of a directory to the list.
 
       The path must exist and must be a directory or a link to a directory.
       The contents of the directory (as well as the directory path itself) will
       be recursively added to the list, subject to any exclusions that are in
-      place.  
+      place.  If you only want the directory and its immediate contents to be
+      added, then pass in C{recursive=False}.
 
       @note: If a directory's absolute path matches an exclude pattern or path,
       or if the directory contains the configured ignore file, then the
@@ -346,13 +351,12 @@ class FilesystemList(list):
       path itself is added to the list once it has been discovered.  It does
       I{not} modify any behavior related to directory recursion.
    
-      @note: The L{excludeDirs} flag only controls whether any given directory
-      path itself is added to the list once it has been discovered.  It does
-      I{not} modify any behavior related to directory recursion.
-
       @param path: Directory path whose contents should be added to the list
       @type path: String representing a path on disk
 
+      @param recursive: Indicates whether directory contents should be added recursively.
+      @type recursive: Boolean value
+
       @return: Number of items recursively added to the list
 
       @raise ValueError: If path is not a directory or does not exist.
@@ -360,9 +364,9 @@ class FilesystemList(list):
       """
       path = encodePath(path)
       path = normalizeDir(path)
-      return self._addDirContentsRecursive(path)
+      return self._addDirContentsInternal(path, recursive=recursive)
 
-   def _addDirContentsRecursive(self, path, includePath=True):
+   def _addDirContentsInternal(self, path, includePath=True, recursive=True):
       """
       Internal implementation of C{addDirContents}.
 
@@ -376,6 +380,7 @@ class FilesystemList(list):
 
       @param path: Directory path whose contents should be added to the list.
       @param includePath: Indicates whether to include the path as well as contents.
+      @param recursive: Indicates whether directory contents should be added recursively.
 
       @return: Number of items recursively added to the list
 
@@ -405,7 +410,10 @@ class FilesystemList(list):
             if os.path.islink(entrypath):
                added += self.addDir(entrypath)
             else:
-               added += self._addDirContentsRecursive(entrypath)
+               if recursive:
+                  added += self._addDirContentsInternal(entrypath)
+               else:
+                  added += self.addDir(entrypath)
       return added
 
 
@@ -748,8 +756,8 @@ class BackupFileList(FilesystemList):
       Python hashing algorithms.
 
       In my tests using a 110 MB file on CD, the original implementation
-      requires 111 seconds while this implementation requires only 40-45
-      seconds, which is a pretty substantial speed-up.  
+      requires 111 seconds.  This implementation requires only 40-45 seconds,
+      which is a pretty substantial speed-up.  
 
       Practice shows that reading in around 4kB (4096 bytes) at a time yields
       the best performance.  Smaller reads are quite a bit slower, and larger
@@ -765,7 +773,7 @@ class BackupFileList(FilesystemList):
       @raise OSError: If the file cannot be opened.
       """
       s = sha.new()
-      f = open(path)
+      f = open(path, mode="rb")  # in case platform cares about binary reads
       readBytes = 4096  # see notes above
       while(readBytes > 0):
          readString = f.read(readBytes)
@@ -812,7 +820,7 @@ class BackupFileList(FilesystemList):
       elif algorithm == "alternate_fit": return alternateFit(table, capacity)[0]
       else: raise ValueError("Algorithm [%s] is invalid." % algorithm);
 
-   def generateTarfile(self, path, mode='tar', ignore=False):
+   def generateTarfile(self, path, mode='tar', ignore=False, flat=False):
       """
       Creates a tar file containing the files in the list.
 
@@ -825,6 +833,10 @@ class BackupFileList(FilesystemList):
       the extra functionality out-weighs the disadvantage of not being
       "standard".
 
+      If you pass in C{flat=True}, then a "flat" archive will be created, and
+      all of the files will be added to the root of the archive.  So, the file
+      C{/tmp/something/whatever.txt} would be added as just C{whatever.txt}.
+
       By default, the whole method call fails if there are problems adding any
       of the files to the archive, resulting in an exception.  Under these
       circumstances, callers are advised that they might want to call
@@ -861,6 +873,9 @@ class BackupFileList(FilesystemList):
       @param ignore: Indicates whether to ignore certain errors.
       @type ignore: Boolean
 
+      @param flat: Creates "flat" archive by putting all items in root 
+      @type flat: Boolean
+
       @raise ValueError: If mode is not valid
       @raise ValueError: If list is empty
       @raise ValueError: If the path could not be encoded properly.
@@ -877,21 +892,26 @@ class BackupFileList(FilesystemList):
          tar.posix = False    # make a GNU-compatible archive without file length limits
          for entry in self:
             try:
-               tar.add(entry, recursive=False)
-            except tarfile.TarError:
+               if flat:
+                  tar.add(entry, arcname=os.path.basename(entry), recursive=False)
+               else:
+                  tar.add(entry, recursive=False)
+            except tarfile.TarError, e:
                if not ignore:
-                  raise
+                  raise e
                logger.info("Unable to add file [%s]; going on anyway." % entry)
             except OSError, e:
                if not ignore:
                   raise tarfile.TarError(e)
                logger.info("Unable to add file [%s]; going on anyway." % entry)
          tar.close()
-      except tarfile.TarError:
+      except tarfile.TarError, e:
+         try: tar.close()
+         except: pass
          if os.path.exists(path): 
             try: os.remove(path) 
             except: pass
-         raise
+         raise e
 
    def removeUnchanged(self, digestMap, captureDigest=False):
       """
@@ -1015,14 +1035,15 @@ class PurgeItemList(FilesystemList):
    # Add methods
    ##############
 
-   def addDirContents(self, path):
+   def addDirContents(self, path, recursive=True):
       """
       Adds the contents of a directory to the list.
 
       The path must exist and must be a directory or a link to a directory.
       The contents of the directory (but I{not} the directory path itself) will
       be recursively added to the list, subject to any exclusions that are in
-      place.  
+      place.  If you only want the directory and its contents to be added, then
+      pass in C{recursive=False}.
 
       @note: If a directory's absolute path matches an exclude pattern or path,
       or if the directory contains the configured ignore file, then the
@@ -1045,6 +1066,9 @@ class PurgeItemList(FilesystemList):
       @param path: Directory path whose contents should be added to the list
       @type path: String representing a path on disk
 
+      @param recursive: Indicates whether directory contents should be added recursively.
+      @type recursive: Boolean value
+
       @return: Number of items recursively added to the list
 
       @raise ValueError: If path is not a directory or does not exist.
@@ -1052,7 +1076,7 @@ class PurgeItemList(FilesystemList):
       """
       path = encodePath(path)
       path = normalizeDir(path)
-      return super(PurgeItemList, self)._addDirContentsRecursive(path, includePath=False)
+      return super(PurgeItemList, self)._addDirContentsInternal(path, includePath=False, recursive=recursive)
 
 
    ##################
