@@ -8,7 +8,7 @@
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
-# Copyright (c) 2004-2005 Kenneth J. Pronovici.
+# Copyright (c) 2004-2006 Kenneth J. Pronovici.
 # All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
@@ -27,7 +27,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: image.py 870 2005-10-01 17:00:57Z pronovic $
+# Revision : $Id: image.py 986 2006-06-24 19:23:22Z pronovic $
 # Purpose  : Provides ISO image-related objects.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -525,7 +525,7 @@ class IsoImage(object):
          if entries[key] is None:
             dirEntries.append(key)
          else:
-            dirEntries.append("%s/=%s" % (entries[key].strip(os.sep), key))
+            dirEntries.append("%s/=%s" % (entries[key].strip("/"), key))
       return dirEntries
    _buildDirEntries = staticmethod(_buildDirEntries)
 
