@@ -11,7 +11,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: release.py 936 2005-12-23 02:44:49Z pronovic $
+# Revision : $Id: release.py 1002 2006-09-04 18:48:13Z pronovic $
 # Purpose  : Provides location to maintain release information.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -31,10 +31,10 @@ Provides location to maintain version information.
 @author: Kenneth J. Pronovici <pronovic@ieee.org>
 """
 
-AUTHOR      = "Kenneth J. Pronovici"
+AUTHOR      = "Kenneth J. Pronovici, et. al"
 EMAIL       = "pronovic@ieee.org"
-COPYRIGHT   = "2004-2005"
-VERSION     = "2.7.2"
-DATE        = "22 Dec 2005"
+COPYRIGHT   = "2004-2006"
+VERSION     = "2.8.1"
+DATE        = "04 Sep 2006"
 URL         = "http://cedar-solutions.com/software/cedar-backup"
 
